/*
 * Mini-projet 1:  stimulus
 * G'en'erateur de nombres pseudo-al'eatoires
 * dans un intervalle donn'e
 *
 * Etienne Wodey -- 2010-08
 * Marco Antognini -- 2011-08 (C++11)
 * Marco Antognini -- 2012-09 (C++11 et suppression de boost)
 */

#ifndef _RAND_RANGE_
#define _RAND_RANGE_

#include <random>
#include <type_traits> // pour std::conditional et std::is_integral

// C++11 : <random>
// ---------------------------------------------------------------------
// A la place de srand/drand/... ou des outils de boost,
// la nouvelle norme met à disposition
// des outils plus complet de génération de nombre aléatoire.
// Un exemple est donné à l'adresse suivante.
// http://en.wikipedia.org/wiki/C%2B%2B0x#Extensible_random_number_facility

template <typename T>
T rand_range(T min, T max)
{
    // Utilisation de "static" ici pour n'avoir qu'un seul générateur
    // de nombre aléatoire et non pas un nouveau à chaque fois que la
    // fonction est appelée. Sinon la distribution serait faussée.
    
    // Il existe de nombreux algorithme pour générer des nombres
    // aléatoire. On aurait pu utiliser Mersenne Twister, aka
    // std::mt19937, ou autre, mais on va utiliser l'algorithme
    // par défaut.
    static std::default_random_engine algo;
    
    // Il faut aussi donner une "graine" à l'algo pour avoir des
    // nombre aléatoire différent entre deux lancement de l'application.
    // Mais comme avec srand, il faut le faire une seule fois.
    static bool initialise = true;
    if (initialise) {
        initialise = false;
        std::random_device rd;
        algo.seed(rd());
    }
    
    // On veut une distribution uniforme sur [min, max] mais en fonction
    // de T on veut générer des entiers ou des réels. On définit donc
    // distribution_type, un alias en fonction de T.
    typedef typename std::is_integral<T> condition; // si T est une sorte d'entier alors
    typedef typename std::uniform_int_distribution<T> integer_dist; // on veut ce type-ci
    typedef typename std::uniform_real_distribution<T> real_dist; // sinon ce type-là
    
    typedef typename std::conditional<condition::value, integer_dist, real_dist>::type distribution_type;
    
    // On déclare une distribution uniforme de nombres entiers/réels.
    distribution_type dist(min, max);
    
    // Et on génère un nombre avec notre algo sur notre distribution.
    return dist(algo);
}

#endif // _RAND_RANGE_

